local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Squared Tower materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SST_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_SST_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_SST_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_SST_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_SST_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_SST_WHITE_STONE_FRAME_MATERIAL")

-- Register Small Squared Tower prefab nodes
StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", StoneGateSetPrefix .. "_SST_TOP_PREFAB")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:registerAsset({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }}
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.05 }
	    }
	})
end

registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TOP")

function registerSmallSquaredTower(_option, _top, _buildingZone)
    -- Register Small Squared Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PREFAB")

    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING1" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING2" .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PREFAB",
		EndPart = _top,
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	},
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
    })
end
function registerSmallSquaredTowerFunction(_option)
    -- Register the stone gate set small squared tower root part
    StoneGate:overrideAsset({
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PART",
        VillagerCapacity = 2,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "SST_ROOT" .. _option)
    })
end

registerSmallSquaredTower("", StoneGateSetPrefix .. "_SST_TOP_PART", _SstBuildingZone) -- with stone ground
registerSmallSquaredTower("P", StoneGateSetPrefix .. "_SST_TOP_PART", _SstBuildingZone) -- with wooden ground
registerSmallSquaredTower("HALFWHITE", StoneGateSetPrefix .. "_SST_TOP_PART", _SstBuildingZone) -- Half white with stone ground
registerSmallSquaredTowerFunction("") -- with stone ground
registerSmallSquaredTowerFunction("P") -- with wooden ground
registerSmallSquaredTowerFunction("HALFWHITE") -- Half white with stone ground


registerSmallSquaredTower("GATE", StoneGateSetPrefix .. "_SST_TOP_PART", _SstGateBuildingZone)   -- Gate with stone ground
registerSmallSquaredTower("GATEP", StoneGateSetPrefix .. "_SST_TOP_PART", _SstGateBuildingZone)  -- Gate with wooden ground
registerSmallSquaredTower("GATE2", StoneGateSetPrefix .. "_SST_TOP_PART", _SstGateBuildingZone)  -- Gate2 with stone ground
registerSmallSquaredTower("GATE2P", StoneGateSetPrefix .. "_SST_TOP_PART", _SstGateBuildingZone) -- Gate2 with wooden ground

-- Roofs...
function registerRoofPart(_nameFbx, _nameAsset, _buildingPartType)
    StoneGate:registerAssetId("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 3*_NoSandbox }
		}}
	}
    })

    StoneGate:registerPrefabComponent("models/roofs/simpleRoofs/roofSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

function registerRoofAnglePart(_angle)
    registerRoofPart( "Roof4Closed" .. _angle,               "ROOF_4CLOSED" .. _angle,                 "MAJOR" )
    registerRoofPart( "Roof1Opened" .. _angle,               "ROOF_1OPENED" .. _angle,                 "MAJOR" )
    registerRoofPart( "Roof2Opposite" .. _angle,             "ROOF_2OPPOSITE" .. _angle,               "MAJOR" )
    registerRoofPart( "Roof2Adjacent" .. _angle,             "ROOF_2ADJACENT" .. _angle,               "MAJOR" )
    registerRoofPart( "Roof3Opened" .. _angle,               "ROOF_3OPENED" .. _angle,                 "MAJOR" )
    registerRoofPart( "Roof4Opened" .. _angle,               "ROOF_4OPENED" .. _angle,                 "MAJOR" )
    registerRoofPart( "RoofExtensionStraight" .. _angle,     "ROOF_EXTENSION_STRAIGHT" .. _angle,      "MINOR" )
    registerRoofPart( "RoofExtensionAdvanced" .. _angle,     "ROOF_EXTENSION_ADVANCED" .. _angle,      "MINOR" )
    registerRoofPart( "Roof3Shed" .. _angle,                 "ROOF_3SHED" .. _angle,                   "MAJOR" )
    registerRoofPart( "Roof2Shed" .. _angle,                 "ROOF_2SHED" .. _angle,                   "MAJOR" )
    registerRoofPart( "RoofShedExtensionStraight" .. _angle, "ROOF_SHED_EXTENSION_STRAIGHT" .. _angle, "MINOR" )
end
registerRoofAnglePart("45")
registerRoofAnglePart("30")
registerRoofAnglePart("60")

---- Ramparts...

function registerRampartPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType, _category)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	}
    })

    StoneGate:registerPrefabComponent("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerRampartPart( "ramparts/rampartSmallSquaredTower", "Rampart4Closed", "RAMPART_4CLOSED", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "Rampart1Opened", "RAMPART_1OPENED", "MAJOR", "EXTENSION" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "Rampart3Opened", "RAMPART_3OPENED", "MAJOR", "EXTENSION" )

-- platforms
StoneGate:configurePrefabFlagList("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", { "PLATFORM" })
